        program bio_2d

c     Global NCAR Two-Dimensional Biological Mixed Layer Model
c
c     version 1.0 (Dec. 14th, 1995)
c     created by S.C. Doney, Climate and Global Dynamics, NCAR
c     (doney@ncar.ucar.edu) (303) 497-1639
c
c   	Version 4.0 June 2000, separate wvel and dhdt for tracking
c         mixed layer budgets
c
c	Version 5.0 Adding phosphate as limiting nutrient, diazotrophs
c		(N-fixers) as 3rd phyto group, CaCO3 production parameterized
c
c***************************************************************************
c     Purpose: Integrate a set of imt x jmt independent biological
c     mixed layer models on a global grid returning the time evolution
c     of biological scalars and limited set of fluxes.
c
c     Model Structure: The model structure is as follows
c
c          main driver (this program) =>
c               integrator (4th order Runge-Kutta) =>
c                     biology driver (combine biology and physics) =>
c                          biology subroutines (user supplied)
c
c     The model is constructed in modular form such that the driver 
c     routine does not know the details of the ecosystem to be modeled. 
c     The biological scalars and parameters are supplied to the driver 
c     as generic arrays and vectors and are passed to the biological driver
c     routine either through the integrator (scalars) or common statements
c     (parameters, forcing). The biological subroutines, which compute the 
c     actual rate of change with time can be replaced if the user so 
c     desires. Note that the required dimension values for the biological 
c     routine need to be set in bioparam_x3pV5.inc
c
c     Scope: The model is constructed to examine biological variability
c     on large spatial (non-eddy resolving) and temporal (monthly) 
c     timescales. As such, the model forcing interpolated with cubic splines
c     from data supplied at monthly. High temporal resolution is NOT
c     currently recommended. The model domain is, at present, also restricted
c     to the mixed layer though this may change with future implementations.
c     
c     Model Components:
c     bio_2d.f              main driver program (bio_2d)
c                                initialization
c                                input/output
c                                splining forcing functions
c                                integration control
c                           bioshell
c                                biological/physical driver
c
c     bio_subs.f            biological subroutines
c                                ecosystem flow rates
c                                particle sinking
c                                vertical fluxes across ML base
c                                light penetration
c
c     numerics_subs.f       numeric subroutines (Numerical Recipies, 1992)
c                                adaptive time-step control
c                                Runge-Kutta integrator
c                                cubic splines
c                                (Note: spline calculation changed to
c                                add derivative calculation).
c                                
c     Biological Parameters:
c     bioparam_x3pV5.inc          grid and dimension parameters
c                           NOTE: parameters must be changed if any 
c                           alterations are made in bio_subs.f
c
c     Common Data:
c     biocommon_x3p.inc         commom biological variables
c
c****************************************************************************
c     Model Dimension and Decleration Statements:
        implicit none

c     Grid Dimensions: (see bioparam_x3pV5.inc)
c     imt         number of grid longtitude points 
c     jmt         number of grid latitude points
c     km          number of vertical grid levels (kmt=1 for mixed layer) 
c     kmt         land mask array (0=land, >0=ocean)
c
c     current grid set to NCAR-MOM x3p configuration (approx 3 deg. x 3 deg.)
c     integer imt,jmt,km
c     parameter(imt=102, jmt=116, km=1) 
        include 'bioparam_x3pV5.inc'
        integer kmt(imt,jmt)    

c     Biological Dimensions: (see bioparam_x3pV5.inc)
c     npar        number of biological parameters
c     nsb         number of biological scalars
c     nflux       number of biological fluxes retained for output
c      integer npar,nsb,nflux
c     parameter(npar=36, nsb=36, nflux=36) 

c     Biological Variables:
c     bio(imt,jmt,nsb)        biological scalar array
c     flux(imt,jmt,nflux)     biological integrated flux array
c     bio_surf(imt,jmt,nsb)   array of surface intercepts for bottom bc
c     bio_slope(imt,jmt,nsb)  array of linear slopes for bottom bc
c     x(nsb+nflux)        scalar and flux vector for integrator for one point
        real bio(imt,jmt,nsb), flux(imt,jmt,nflux)
        real bio_surf(imt,jmt,nsb), bio_slope(imt,jmt,nsb)
        real x(nsb+nflux)

c     Biological Common Variables:
c     those variables including the forcing and biological parameters
c     required by the biological subroutines but which are not passed
c     directly by the integrator.
c     biopar(npar)        biological parameter values
c     bioname(nsb)        names of biological scalars
c     parname(npar)       names of biological parameters
c     fluxname(nflux)     names of biological fluxes
c     bbc(nsb)            bottom boundary condition
c     surf(nsb)           surface intercept for bottom boundary condition
c     slope(nsb)          linear slope for bottom boundary condition
c     real biopar(npar)
c     see also cubic spline forcing data below
c     character*20 bioname(nsb),parname(npar),fluxname(nflux)
c     real surf(nsb), slope(nsb), bbc(nsb)
        include 'biocommon_x3p.inc'

c     Forcing Variables: 
c     nforc       number of evenly spaced forcing intervals in year
c     mforc       number of forcing variables (in bioparam_x3pV5.inc)
c     qsw         surface shortwave flux (W/m2)
c     sst         sea surface temp (deg C)
c     hmix        mixed layer depth (m)
c     wturb       turbulent exchange rate across base of ML (m/d)
c     wvel        mean vertical velocity across base of ML (m/d)
c     fice        ice fraction (0.0-1.0)
c     feflux      Fe deposition flux (mmolFe/m2/day), 12 monthly fields
c     forc        storage array for cubic splines, values  (in biocommon_x3p.inc)
c     tforc       time vector for cubic splines (days)
c     cubspl      storage array for cubic spline  2nd derivative
c        integer nforc,mforc
c        parameter (nforc=12,mforc=7)          
        real qsw(imt,jmt,nforc), sst(imt,jmt,nforc), 
     $       hmix(imt,jmt,nforc), wturb(imt,jmt,nforc),
     $       wvel(imt,jmt,nforc), 
     $       fice(imt,jmt,nforc), feflux(imt,jmt,nforc)
c        real forc(2*nforc,mforc),cubspl(2*nforc,mforc)  !in biocommon_x3p.inc
c        real tforc(2*nforc)

c     Time Control Variables (see biocommon_x3p.inc)
c     nspinup          number of spinup years
c     nsave            number of data stores/year
c        integer nsave,nspinup
        real time0,time1,dt

c     Runge-Kutta Integrator:
c     nmax                  maximum number of functions (nmax > nsb + nflux)
c     kmaxx                 maximum number of stored intermediate steps
c     xp(kmaxx)             intermediate storage vector for integrator 
c     yp(nmax,kmaxx)        intermediate storage array for integrator
c     kmax                  number of stored intermediate steps
c     dxsav                 minimum time between intermediate storage
c     h1                    first guess for step size
c     hmin                  minimum step size
c     eps                   fractional accuracy tolerance
c     bioshell              name of external biological driver subroutine 
c     rkqs                  name of external subroutine for actual
c                               Runge-Kutta routine
        integer nmax,kmaxx
        parameter (nmax=75,kmaxx=300)
        real xp(kmaxx),yp(nmax,kmaxx)
        integer kmax,kount,nok,nbad
        real dxsav,eps,h1,hmin
        common /path/ kmax,kount,dxsav,xp,yp
        external bioshell,rkqs

c     Assorted Parameters: (also see bioparam_x3pV5.inc)
c     spd        seconds per day
c     dpy        days per year
c     real spd, dpy
c     parameter(spd=86400., dpy=365.)
c     hml        mixed layer depth
c     i          longitude grid index
c     j          latitude grid index
c     mo         month index
c     iyr        year index
c     k          scalar index
c     n          flux index
c     iun,jun,tun    unit numbers
c     ltest      flag for writing diagnostics for single grid point to screen 
        integer i,j,mo,iyr,k,kk,n,iun,jun,tun,nint,ii
        real dum
c        real hml,dhdt
c        real ti,tj
        
c        logical ltest

c     Input/Output Variables
c     pname      pathname (directory for input data)
c     pname2     pathname (directory for output data)
c     fname      filename
        character pname*30, pname2*30,fname*49

c*******************************************************************
c     Read Input File for Simulation
        write(6,*) 'NCAR 2-D Global Biological Mixed Layer Model'
        write(6,*)
        write(6,*) 'grid size is:', imt,' by ',jmt
        write(6,*) 'number of scalars: ',nsb
        write(6,*) 'number of fluxes: ',nflux
        write(6,*) 'number of parameters: ',npar
        write(6,*) 'number of forcing intervals/year: ',nforc
        write(6,*) 'number of forcing variables: ',mforc
        write(6,*) '   corrections to parameter values should be ',
     &       'made in bioparam_x3pV5.inc!!'

        iun=25   
        jun=26
        tun=27
        write(6,*)
	fname='input_x3p.bio_2dV5'
        call input(iun,fname)

c*******************************************************************
c     Read in Forcing Data Arrays - NOTE THESE DIRECTORY STRINGS AND
c	THE LIKELY THE LENGTH OF THE STRINGS WILL NEED TO BE CHANGED
c	TO CORRESPOND TO YOUR LOCAL DIRECTORY STRUCTURE.
c
        pname='/data/jkmoore/t_dir_/model_ml/'
        pname2='/data/jkmoore/t_dir_/model_ml/'
        write(6,*)
        write(6,*) 'Data i/o pathname is: ',pname
        write(6,*) 'NOTE: length of pname character string must ',
     &       'match path name exactly'

c     sea surface temperature (sst)
	fname='woa98_sst_surf_m_x3p.dat'
        open (iun,file=pname//fname,status='old')
        write(6,*)
        write(6,*) 'reading sea surface temperature from ',fname
        do mo=1,nforc
           read(iun,*) ((sst(i,j,mo),i=1,imt),j=1,jmt)
        enddo
        close(iun)

c     shortwave insolation (qsw)
        fname='qsw_x3p.dat'
        open (iun,file=pname//fname,status='old')
        write(6,*) 'reading shortwave insolation from ',fname
        do mo=1,nforc
           read(iun,*) ((qsw(i,j,mo),i=1,imt),j=1,jmt)
        enddo
        close(iun)

c     mixed layer depth (hmix)
        fname='hmix_levpd_min25_x3p.dat'
        open (iun,file=pname//fname,status='old')
        write(6,*) 'reading mixed layer depth from ',fname
        do mo=1,nforc
           read(iun,*) ((hmix(i,j,mo),i=1,imt),j=1,jmt)
        enddo
        close(iun)

c     turbulent exchange velocity (wturb)
        fname='wturb_x3pV5.dat'
        open (iun,file=pname//fname,status='old')
        write(6,*) 'reading turbulent exchange velocity from ',fname
        do mo=1,nforc
           read(iun,*) ((wturb(i,j,mo),i=1,imt),j=1,jmt)
        enddo
        close(iun)

c     vertical velocity (wvel)
	fname='wvel_x2p_3x3_x3p.dat'
        open (iun,file=pname//fname,status='old')
        write(6,*) 'reading vertical velocity from ',fname
        do mo=1,nforc
           read(iun,*) ((wvel(i,j,mo),i=1,imt),j=1,jmt)
        enddo
        close(iun)

c     ice fraction (fice)
	fname='fice_1098_0999_x3p.dat'
        open (iun,file=pname//fname,status='old')
        write(6,*)
        write(6,*) 'reading ice fraction from ',fname
        do mo=1,nforc
           read(iun,*) ((fice(i,j,mo),i=1,imt),j=1,jmt)
        enddo
        close(iun)

c     fe flux (feflux)
        fname='fedepo_nmol_TegenFung_x3p.dat'
c	fname='fedepo_nmol_Mahowald_x3p.dat'
c	fname='fedepo_nmol_Maho_LGM_x3p.dat'
c	fname='fedepo_fereplete_x3p.dat'
        open (iun,file=pname//fname,status='old')
        write(6,*)
        write(6,*) 'reading Fe flux from ',fname
        do mo=1,nforc
           read(iun,*) ((feflux(i,j,mo),i=1,imt),j=1,jmt)
        enddo
        close(iun)

c     initial conditions
        fname='bio0_x3p_oldV5.dat'
        open (iun,file=pname//fname,status='old')
        write(6,*) 'reading initial conditions from ',fname
        do k=1,nsb
           read(iun,*) ((bio(i,j,k),i=1,imt),j=1,jmt)
        enddo
        close(iun)

c     bottom boundary conditions (surf)
        fname='bio_surf_x3pV5.dat'
        open (iun,file=pname//fname,status='old')
        write(6,*) 'reading boundary condition intercept from ',fname
        do k=1,nsb
           read(iun,*) ((bio_surf(i,j,k),i=1,imt),j=1,jmt)
        enddo
        close(iun)

c     bottom boundary conditions (slope)
        fname='bio_slope_x3p_globalV5.dat'
        open (iun,file=pname//fname,status='old')
        write(6,*) 'reading boundary condition slope from ',fname
        do k=1,nsb
           read(iun,*) ((bio_slope(i,j,k),i=1,imt),j=1,jmt)
        enddo
        close(iun)

c     land mask array (kmt)
        fname='x3p_mask'
        open (iun,file=pname//fname,status='old')
        write(6,*) 'reading file ',fname
        read(iun,*) kmt
        close(iun)      

c************************************************************************
c     Open Output Files
        fname='bio_out_scalar5_x3p0wk.dat'
        open (iun,file=pname2//fname,status='unknown')
        write(6,*) 'writing scalar output to ',fname

        fname='bio_out_flux5_x3p0wk.dat'
        open (jun,file=pname2//fname,status='unknown')
        write(6,*) 'writing flux output to ',fname

c   optional output file used for diagnostics,
c	fname='bio_out_screen5_x3p1.dat'
c	open (tun,file=pname2//fname,status='unknown')
c	write(6,*) 'writing screen output to ',fname
c************************************************************************
c     Integrate model forward in time using Runge-Kutta routines
c     each individual grid point is integrated independently

c     set times for forcing in days; pad time with half year on each end 
        do mo=1,2*nforc
           tforc(mo)=(float(mo-nforc/2)-0.5)*dpy/float(nforc)
        enddo

c     define Runge Kutta Parameters
       kmax=kmaxx
       dxsav=0.1
       eps=5.0e-2
       h1=1.0

       hmin=0.0
       write(6,*)
c**************************************************************************
c     Spin-up up each grid point for nspinup years then run single
c     year saving data at nsave intervals

       do iyr=1,nspinup+1
          if(iyr.le.nspinup) then
             write(6,*) 'Spinup year ',iyr
             nint=4
             dt=dpy/4.0
          else
             write(6,*) 'Simulation year---diagnostics saved at ',
     &            nsave,' even intervals'
             nint=nsave
             dt=dpy/float(nsave) 
          endif

c      loop over number of integration times 
          do ii=1,nint
             time0=(ii-1)*dt
             time1=time0+dt

             write(6,*) iyr,ii,time0,time1

c     loop over all grid points
             do i=1,imt           !imt
              do j=1,jmt         !jmt

c   As there is no lateral advection the grid selection can be modified 
c	for a selected region, i.e.
c 	      do i=63,63
c 	       do j= 56,56
c   or the model can be run at only one location.
 	       

c     check land mask, only integrate ocean points
                   if(kmt(i,j).gt.0) then

c     load forcing data and compute cubic splines; 
c     cubic splines are defined over timespan of -6 months 
c     to + 18 months to insure continuous function at wrap point
                      do mo=1,2*nforc
                         if(mo.le.nforc/2) then
                            kk = mo + nforc/2 
                         elseif(mo.le.nforc*3/2) then
                            kk = mo - nforc/2
                         else
                            kk = mo - nforc*3/2
                         endif
                         forc(mo,1)=qsw(i,j,kk) ! solar insolation (W/m2)    
                         forc(mo,2)=sst(i,j,kk) ! sea surface temp (deg C)
                         forc(mo,3)=hmix(i,j,kk) ! mixed layer depth (m)
                         forc(mo,4)=wturb(i,j,kk) ! turbulent exchange (m/d)
                         forc(mo,5)=wvel(i,j,kk) ! vertical velocity (m/d)
                         forc(mo,6)=fice(i,j,kk) ! ice fraction (0-1)
                         forc(mo,7)=feflux(i,j,kk) ! Fe flux (mmol/m2/day)
                      enddo
                   
c     set "natural" boundary conditions for spline
                      dum=1.e35
                      do kk=1,mforc
                         call spline(tforc,forc(1,kk),2*nforc,
     &                        dum,dum,cubspl(1,kk))
                      enddo
                   
c     load variables and bottom boundary condition slope and intercept
c     into individual grid point vectors for integrator
                      do k=1,nsb
                         x(k) = bio(i,j,k)
                         slope(k) = bio_slope(i,j,k)
                         surf(k) = bio_surf(i,j,k)
                      enddo		      

c     zero accumulated fluxes, by setting = 0.0001, this is subtracted off later
c     accumulated fluxes are stored in the x vector from 
c     (nsb+1) to (nsb+nflux). The integrator treats the
c     flux as simply another variable to integrate, i.e.
c     accumulated flux = int ( flux(t) dt )
                      do k=1,nflux
                         x(k+nsb) = 0.0001
                      enddo

c  optional output lines for diagnostics, etc...                    
c             write(tun,*),i,j,iyr, ii  
c  	   if(iyr.le.nspinup) then     
c             write(tun,*),'phytoN zoo nit detr chl iron phytoC phytoFe'
c             write(tun,*),(x(k),k=1,nsb)
c           endif  
c             write(6,*),i,j,iyr
c             write(6,*),(x(k),k=1,(nsb))


c     integrate model forward in time from time0 to time1
c     store intermediate state variable values in yp                   
                      call odeint(x,nsb+nflux,time0,time1,eps,h1,
     &                     hmin,nok,nbad,bioshell,rkqs)

c     load new state variables and accumulated fluxes into 
c     2-d arrays bio and flux


                      do k=1,nsb
                         bio(i,j,k)=x(k)
                      enddo

                      do n=1,nflux
                         flux(i,j,n)=x(n+nsb) - 0.0001
                      enddo

c  optional output lines          
c          if(iyr.le.nspinup) then            
c            write(tun,*),(x(k),k=1,(nsb))
c  	     write(tun,*),(x(k),k=(nsb+1),(nsb+nflux))
c	     write(tun,*)
c	   endif	     
c            write(6,*),(x(k),k=1,(nsb))
c  	     write(6,*),(x(k),k=(nsb+1),(nsb+nflux))
               
c    write diagnostics for single point
c                      ltest=.FALSE.
c                      if(ltest) then
c                         write(6,*) '*********'
c                         write(6,*) 'nok     nbad     kount'
c                         write(6,*) nok,nbad,kount
c     compute mixed layer depth for inventory check
c                         call splint2(tforc,forc(1,3),cubspl(1,3),
c     &                     2*nforc,xp(kount),hml,dhdt)
c                         write(6,*) xp(kount),hml,dhdt
c                         write(6,*) (yp(k,kount),k=1,nsb)
c                         write(6,*) (yp(k,kount),k=nsb+1,nsb+nflux)
c                      endif
                   endif
                enddo

c optional output line that can be used to watch progress of run                
c             write(6,*),i,j,' End of Column, iyr ', iyr
             enddo

c Output scalars and fluxes after spinup year
             if(iyr.gt.nspinup) then
                do k=1,nsb
                   write(iun,*) ((bio(i,j,k),i=1,imt),j=1,jmt)
                enddo
                do k=1,nflux
                   write(jun,*) ((flux(i,j,k),i=1,imt),j=1,jmt)
                enddo
             endif
          enddo
       enddo



       close(iun)
       close(jun)
c       close(tun)
       write(6,*) 'Run completed.'
       stop
       end

c ==========================================================================
       subroutine input(iun,fname)
c
c     Read input file for 2-D Mixed Layer Model from unit iun
c     file name fname
c     Output returned to main program via biocommon_x3p.inc)
c
        implicit none
        include 'bioparam_x3pV5.inc'
        include 'biocommon_x3p.inc'
        character*42 fname
        integer iun,k,n

        open(iun,file=fname,status='old')

c     Read time control parameters
        read(iun,*)
        read(iun,*)
        read(iun,*)
        read(iun,*) nspinup, nsave
        write(6,*) 'number of spinup years',nspinup
        write(6,*) 'number of data stores / year',nsave
        write(6,*) 'number of forcing variables: ',mforc

c     Read names of biological variables
        read(iun,*)
        read(iun,*)
        read(iun,*)
        write(6,*) 'biological variables: '
        do k=1,nsb
           read(iun,'(a20)') bioname(k)
           write(6,*),bioname(k)
        enddo

c     Read names of biological fluxes
        read(iun,*)
        write(6,*) 'biological fluxes: '
        do k=1,nflux
           read(iun,'(a20)') fluxname(k)
           write(6,*),fluxname(k)
        enddo

c     Read parameter values and names
        read(iun,*)
        write(6,*) 'parameter values and names: '
        do n=1,npar
           read(iun,*) biopar(n),parname(n)
           write(6,*),biopar(n),parname(n)
        enddo

        return
        end
c ==========================================================================

c ==========================================================================
        subroutine bioshell(time,X,dxdt)
c
c     Shell program for biological routines called by integrator.
c
c     bioshell:
c              -computes all of the forcing values required by the
c               main biological program at time t
c              -calls biological routine

        implicit none

c     Model parameters and common data
        include 'bioparam_x3pV5.inc'
        include 'biocommon_x3p.inc'

c     Biological Variables
c     x(nsb+nflux)        scalar and flux vector for single point
c     dxdt(nsb+nflux)     rate of change vector for integrator for one point
c     f_surf    used to set bio_surf as fraction of surf, 
c			function of surf val. and MLdepth

        real x(nsb+nflux), dxdt(nsb+nflux),f_surf

        real time,dydx
        real qsw, sst, hmix, wturb, wvel, dhdt, fice, feflux
        integer k

c     compute forcing data for time using cubic splines
c     Note cubic spline routine modified to also return
c     first derivative (SCD 12/20/95)
        call splint2(tforc,forc(1,1),cubspl(1,1),2*nforc,time,
     *       qsw,dydx)
        call splint2(tforc,forc(1,2),cubspl(1,2),2*nforc,time,
     *       sst,dydx)
        call splint2(tforc,forc(1,3),cubspl(1,3),2*nforc,time,
     *       hmix,dhdt)
        call splint2(tforc,forc(1,4),cubspl(1,4),2*nforc,time,
     *       wturb,dydx)
        call splint2(tforc,forc(1,5),cubspl(1,5),2*nforc,time,
     *       wvel,dydx)
        call splint2(tforc,forc(1,6),cubspl(1,6),2*nforc,time,
     *       fice,dydx)
        call splint2(tforc,forc(1,7),cubspl(1,7),2*nforc,time,
     *       feflux,dydx)

c     ice fraction
        fice=min(fice,1.0)
        fice=max(fice,0.0)

c  adjust incoming shortwave radiation by 6% for albedo and ice cover
        qsw = 0.94*qsw*(1.0-fice)


c     compute bottom boundary condition for scalars from linear background
        hmix = max(hmix,25.0)

c     bbc is a function of ML value and ML depth, when mixed layer is
c     shallow, surf=0.75 * ML value at 25m, decreases as mixed layer deepens
c     goes to 0.0 at 100m, apply to all scalars except, no3, si, & iron
	f_surf = max(0.0, ((25.0/hmix) - 0.25))
	

        do k=1,nsb
           if ((k.EQ.1).OR.(k.EQ.3).OR.(k.EQ.4).OR.(k.EQ.24)) then
              bbc(k) = surf(k) + hmix*slope(k)
           else
              bbc(k) = x(k) * f_surf
           endif
        enddo
       
c set maximum bbc for iron, silicate, nitrate, and phosphate

	 bbc(3)=min(2000.0,bbc(3))
         bbc(4)=min(120.0,bbc(4))	 
         bbc(1)=min(32.0,bbc(1))
         bbc(24)=min(2.0,bbc(24))

c       Guard against any of the scalars going negative due to changes made
c       to the derivative in rkck. DMG
        do k=1,nsb
           x(k)=max(x(k),1.0e-9)
        end do

c     compute time derivatives dxdt for biology 
c
        call biomain(x,dxdt,biopar,bbc,nsb,nflux,npar,
     &       qsw,sst,hmix,dhdt,wturb,wvel,feflux)
c ==========================================================================
   
        RETURN
        END
