        subroutine biomain(x,dxdt,biopar,bbc,nsb,nflux,npar,
     &       qsw,sst,hmix,dhdt,wturb,wvel,feflux)


c     Global NCAR Two-Dimensional Biological Mixed Layer Model
c
c  MAIN BIOSUBS ROUTINE FOR GLOBAL RUNS
c
c     version 1.0 (Dec. 14th, 1995)
c     created by S.C. Doney, Climate and Global Dynamics, NCAR
c     (doney@ncar.ucar.edu) (303) 497-1639
c
c     Version 5.0 (January 2001), by J.Keith Moore and Scott Doney, NCAR
c	(jkmoore@ucar.edu, doney@ucar.edu)
c   Detailed description of the ecosystem model and results are given 
c     in the two Moore et al. references listed below.
c	includes growth model of Geider et al., 1998, (GD98)
c	variable elemental ratios within biota
c	3 phytoplankton classes = small phyto, diatoms, diazotrophs
c	parameterization of calicification by coccolithophores
c	atmospheric mineral dust source for dissolved iron and silicate
c
cReferences
c META	Moore, J.K., Doney, S.D., Kleypas, J.A., Glover, D.M., Fung, I.Y.,
c       2001. An intermediate complexity marine ecosystem model for the 
c	Global Domain. Deep Sea Res. II, in press.
c
c METB  Moore, J.K., Doney, S.D., Glover, D.M., Fung, I.Y., 2001. Iron
c	cycling and nutrient limitation patterns in surface waters of 
c	the world ocean. Deep Sea Res. II, in press.
c
c	Geider, R.J., MacIntyre, H.L., Kana, T.M., 1998. A dyanamic 
c	regulatory model of phytoplankton acclimation to light, nutrients,
c	and temperature. Limnol. Oceanogr., 43: 679-694.
c
c************************************************************************
c     Purpose: Compute time derivatives resulting from biological and 
c     physical processes for individual mixed layer model
c
c     This code should be modified to fit individual applications
c                                                                72C
c***********************************************************************
c     External Variables
c     biological dimensions: 
c     npar        number of biological parameters
c     nsb         number of biological scalars
c     nflux       number of biological fluxes retained for output
	implicit none
        integer npar,nsb,nflux     

c     biological variables:
c     x(nsb+nflux)        scalar and flux vector from integrator for one point
c     dxdt(nsb+nflux)     rate of change vector for integrator for one point
c     bbc(nsb)            bottom boundary conditions
c     biopar(npar)        biological parameter list
        real x(nsb+nflux), dxdt(nsb+nflux), biopar(npar), bbc(nsb)

c     units: rates are typically in 1/day

c     forcing variables: 
c     qsw         surface shortwave flux (W/m2)
c     sst         sea surface temp (deg C)
c     hmix        mixed layer depth (m)
c     wturb       turbulent exchange rate across base of ML (m/d)
c     dhdt        entrainment/detrainment rate across base of ML (m/d)
c                    due to variations in mixed layer depth
c     wvel        vertical velocity at base of mixed layer (m/d)
c     feflux      Fe atmos. deposition flux (nmol/m2/day)
c		  in the code this is converted to Si flux using 
c                 assuming a constant Si/Fe ratio in the dust
c
        real qsw,sst,hmix,dhdt,wturb,wvel,feflux

c****************************************************************************
c     Local variables
c
c     biological variables:
c     no3         nitrate  (mmol N /m3)
c     amm         ammonia (mmol N / m3)
c     iron        dissolved iron (nmol/m3)
c     silica      dissolved si (mmol Si/m3)
c     po4	phosphate (mmol P/m3)
c     sphytoC     small phyto C (mmol C/m3)
c     sphytoN     small phyto N (mmol N/m3)
c     sphytoFe    small phyto Fe (nmol Fe/m3)
c     sphytoChl   small phyto Chl (mg Chl/m3)
c     sphytoCaco3 small phyto CaCO3 (mmol C /m3)
c     sphytoP	small phyto P (mmol P/m3)
c     lphytoC     large phyto C (mmol C/m3)
c     lphytoN     large phyto N (mmol N/m3)
c     lphytoFe    large phyto Fe (mmol Fe/m3)
c     lphytoChl   large phyto chl (mg Chl/m3)
c     lphytoSi    large phyto silica (mmol Si/m3)
c     lphytoP   large phyto P (mmolP/m3)
c     zooC      zooplankton C (mmol C/m3)
c     zooN      zooplankton N (mmol N/m3)
c     zooFe     zooplankton Fe (nmol Fe/m3)
c     zooP	zooplankton P (mmol P/m3)
c     ldetrC      large detritus C (mmol C/m3)
c     ldetrN      large detritus N (mmol N/m3)
c     ldetrFe     large detritus Fe (nmol Fe/m3)
c     ldetrSi     large detritus Si (mmol Si/m3)
c     ldetrCaco3 large detrital caco3 (mmol C /m3)
c     ldetrP    large detrital P (mmol P/m3)
c     sdetrC      small detritus C (mmol C/m3)
c     sdetrN      small detritus N (mmol N/m3)
c     sdetrFe     small detritus Fe (nmol Fe/m3)
c     sdetrP    small detrital P (mmol P/m3)
c     diazC     diazotroph C (mmol/m3)
c     diazN     diazotroph N (mmol/m3)
c     diazFe    diazotroph Fe (nmol/m3) 
c     diazChl   diazotroph Chl (mg/m3)
c     diazP	diazotroph P (mmol/m3)

	real no3,amm,iron,silica,sphytoC,sphytoN,sphytoFe,sphytoChl
	real lphytoC,lphytoN,lphytoFe,lphytoChl,lphytoSi,zooC
	real zooN, zooFe, ldetrC,ldetrN,ldetrFe,ldetrSi,sdetrC
	real sdetrN,sdetrFe,po4,sphytoP,lphytoP,zooP,ldetrP,sdetrP
	real diazC,diazN,diazFe,diazChl,diazP
	real sphytoCaco3, ldetrCaco3

c     biological parameters
c       Many of the parameters are taken from GD98 and have similar names,
c	see META for details.  Note some parameters are set in the input
c       file and others within the code itself.

c     Qmin        min. phyto N:C ratio (mol/mol)
c     Qmax        max. phyto N:C ratio (mol/mol)
c     PCref       maximum phyto C-specific growth rate at tref (GD98) (per day)
c     z_umax      max. zoopl grazing rate on sphyto at tref (per day)
c     lz_umax     max. zoopl grazing rate on lphyto at tref (per day)
c     dt_umax     max. zoopl grazing rate on large detritus
c     dz_umax     max. zoopl grazing rate on diazotrophs
c     sp_mort     small phyto non-zoo plankton death, excretion,lysis,etc. (%/day)
c     sp_mort2    phytop quadratic mortality rate, aggregation
c     lp_mort     large phyto non-grazing death rate
c     lp_mort2    large phyto quadratic moratility rate, aggregation/sinking
c     z_mort      zoopl linear mortality rate (%/day)
c     z_mort2     zoopl quadratic mortality rate, to higher trophic levels
c     sd_remin    small  detrital remineralization rate (%/day)
c     ld_remin    large detrital remineralization rate (%/day)
c     ld_sink     large detritial sinking rate   (m/day)
c     sp_kno3     small phyto nitrate uptake half saturation coeff. (mmol N/m3)
c     lp_kno3     large phyto nitrate uptake half saturation coeff. (mmol N/m3)
c     sp_kamm     small phyto ammonia uptake half saturation coeff. (mmol N/m3)
c     lp_kamm     large phyto ammonia uptake half saturation coeff. (mmol N/m3)
c     sp_kiron    small phyto iron uptake half saturation coefficient (nmol FE/m3)
c     lp_kiron    large phyto iron uptake half saturation coefficient (nmol FE/m3)
c     lp_ksilica  large phyto si uptake half saturation coefficient (mmol Si/m3)
c     z_grz       grazing coefficient for small phyto and diazotrophs 
c     z_ingest    zooplankton ingestion coefficient
c     firon       fraction of atmos. iron deposition which dissolves
c     Vncref      value of Vnc (carbon specific N-uptake rate) 
c			(mmolN/mmolC) at Tref (GD98)
c     thetaNmax   max. value of thetaN (chl/N ratio) (mgChl/mmolN)
c     lambda_no3  cost of biosynthesis (mmolC/mmolN) (GD98)
c     Qfemax      max. fe/C ratio, currently input as parameter
c     Qfemin      min. fe/C ratio, currently input as parameter
c     Qsimin      large phyto min si/C ratio
c     Qsimax      large phyto max si/C ratio
c     alphachl    chl. specific initial slope of P_I curve (GD98)
c                      units are (mmolC m^2/mgChl W day)
c     diaz_mort   mortality/respiration loss for diazotrophs (%/day)
c     sp_kpo4     half-saturation constant for P uptake by small P and diazotrophs
c     lp_kpo4     half-saturation for P uptake by diatoms
c     alphadiaz   initial slope of P_I curve for diazotrophs


        real Qmin,Qmax,PCref, z_umax, lz_umax, dt_umax, dz_umax
        real sp_mort,sp_mort2,lp_mort,lp_mort2, z_mort,z_mort2,sd_remin
        real ld_remin,ld_sink,sp_kno3,lp_kno3,sp_kamm,lp_kamm,lp_ksilica
        real sp_kiron,lp_kiron, z_grz,z_ingest,firon
        real Vncref,thetaNmax, lambda_no3,Qfemax,Qfemin
        real Qsimin,Qsimax,alphachl,diaz_mort,sp_kpo4,lp_kpo4,alphadiaz

c     local biological variables
c     Epar        average PAR over mixed layer depth (W/m2)
c     kpar        absorption coefficient for PAR (m)
c     thetaC_sp   small phytoplankton chl to C ratio (mg/mmol)
c     thetaN_sp   small phytoplankton chl to N ratio (mg/mmol)
c     pChl        Chl synthesis regulation term (dimensionless)
c     PCphoto_sp  small C-specific rate of photosynthesis (per day)
c     thetaC_lp   large phytoplankton chl to C ratio (mg/mmol)
c     thetaN_lp   large  phytoplankton chl to N ratio (mg/mmol)
c     PCphoto_lp  large phyto C-specific rate of photosynthesis (per day)
c     PCmax       max value of PCphoto at temperature sst (per day)
c     thetaC_diaz diazotroph chl/C ratio
c     thetaN_diaz diazotroph chl/N ratio
c     PCphoto_diaz diazotroph C-specific rate of photosynthesis (per day)
c     Q_diaz      diaz N cell quota (N/C ratio)
c     uQ_diaz
c     uQfe_diaz
c     Vnc_diaz    diaz C-specific Nitrogen fixation rate (mmolN/mmolC/day)
c     Vncmax_diaz max. rate at current temperature 
c     diaz_Nexcrete excretion of fixed N (30% of total N fixation)
c     Qfec_diaz   diaz fe/C ratio (nmol/mol)
c     Vfec_diaz   C-specific iron uptake (nmolFe/mmolC/day)
c     Vfecmax_diaz temperature dependent max. C-specific iron uptake (nmolFe/mmolC/day)
c     Vfecref_diaz
c     f_nit_diaz  diaz fractional factor for quota
c     f_fe_diaz   diaz fractional factor for quota
c     f_p_diaz    diaz fractional factor for quota
c     f_p_sp      small phyto fractional factor for quota
c     f_p_lp      fractional factor for quota
c     uQp_diaz    boundary check for Q used in uptake equations
c     uQp_sp      boundary check for Q used in uptake equations
c     uQp_lp      boundary check for Q used in uptake equations
c     photoacclim_diaz  Chl synthesis term used in photoadaptation (GD98)
c     Qp_sp       small phyto P/C ratio
c     Qp_lp       large phyto P/C ratio
c     Qp_diaz     diaz P/C ratio
c     zooQp       zooplankton P/C ratio (mmol/mmol)
c     sdetrQp     small detritus P/C ratio
c     ldetrQp     large detritus P/C ratio
c     Vpo4ref     max. po4 uptake at ref. temp = PCref * Qpo4max
c     Vpo4max     max. po4 uptake at current temperature
c     PCref_diaz  max. growth rate for diaz 
c     Vpo4ref_diaz max. po4 uptake at ref. temp + PCref_diaz * Qpo4max_diaz
c     Qpo4max     max. P/C ratio for small phyto and large phyto
c     Qpo4min     min. P/C ratio for small phyto and large phyto
c     Qpo4max_diaz max. P/C ratio for diaz
c     Qpo4min_diaz min. P/C ratio for diaz	
c     Q_sp        small phyto cell quota of N, (phyto N:C ratio)
c     Q_lp        large phyto cell quota of N, (phyto N:C ratio)
c     Tfunc       temperature response function GD98
c     Vnc_sp      small phyto C-specific nitrogen uptake rate (mmolN/mmolC/day)
c     Vnc_lp      large phyto C-specific nitrogen uptake rate (mmolN/mmolC/day)
c     Vncmax_sp   max Vnc & Vammc at temperature sst (mmolN/mmolC/day)
c     Vncmax_lp   max Vnc & Vammc at temperature sst (mmolN/mmolC/day)
c     Qfec_sp     small phyto iron quota, ( fe/C ratio, nmolfe/mmolC)
c     Qfec_lp     large phyto iron quota, ( fe/C ratio, nmolfe/mmolC)
c     f_nut       nutrient limitation factor which modifies C fixation
c     Vfec_sp     small phyto C-specific iron uptake (nmolFe/mmolC/day)
c     Vfec_lp     large phyto C-specific iron uptake (nmolFe/mmolC/day)
c     Vfecref     max. Vfec at ref. temperature, = PCref * Qfecmax (GD98)
c     Vfecmax_sp  max. Vfec at temperature sst (nmolFe/mmolC/day)
c     Vfecmax_lp  max. Vfec at temperature sst (nmolFe/mmolC/day)
c     f_nit_sp    fractional factor for small phyto nitrogen quota
c     f_fe_sp     fractional factor for small phyto fe quota
c     f_nit_lp    fractional factor for large phyto nitrogen quota
c     f_fe_lp     fractional factor for large phyto fe quota
c     uQ_sp       boundary check for Q used in N uptake equations
c     uQfe_sp     boundary check for Qfe used in Fe uptake equations
c     uQ_lp       boundary check for Q used in N uptake equations
c     uQfe_lp     boundary check for Qfe used in Fe uptake equations
c     V1_sp       small phyto nitrate uptake rate
c     V2_sp       small phyto ammonia upatke rate (V1 + V2) <= 1.0
c     photoacclim_sp  chl synthesis term used in photoadaptation (GD98)
c     V1_lp       large phyto nitrate uptake rate
c     V2_lp       large phyto ammonia upatke rate (V1 + V2) <= 1.0
c     photoacclim_lp  chl. synthesis term used in photoadaptation (GD98)
c     nitrif      nitrification (amm -> no3), only at low light
c     zooQ        zooplankton N/C ratio
c     zooQfe	  zooplankton fe/C ratio
c     sdetrQ      small detritus N/C ratio
c     ldetrQ      large detritus N/C ratio
c     ldetrQsi    large detritus Si/C ratio
c     fe_scavenge loss of dissolved iron due to particle scavenging
c     Pprime      used to limit phyto mortality at very low biomass
c     Qsi_lp      large phyto silica quota (mmol Si/mmol C)
c     uQsi_lp	  used in bounds check for si uptake
c     Vsicref     max. Vsic at ref. temp. = PCref * Qsicmax
c     Vsicmax     max. Vsic at current sst
c     Vsic_lp     C-specific Si uptake for large phyto
c     f_si_lp     large phyto fraction si quota
c     Zprime      used to decrease mortality as biomass gets very low
c     f_zoo_detr  fraction of zoo losses into large detrital pool
c     amm_V_sp    ammonia uptake by small phyto
c     no3_V_sp    nitrate uptake by small phyto
c     amm_V_lp    ammonia uptake by large phyto
c     no3_V_lp    nitrate uptake by large phyto
c     reduceV     factor used in nutrient uptake
c     Qfemax_diaz max. fe/C ratio for diazs
c     Qfemin_diaz min. fe/C ratio for diazs
c     Vncref_diaz max. N-fixation rate at Tref = PCref_diaz * Qmax * 1.43
c	  it is assumed that 30% more N is fixed than needed, see META
c     photoFe_diaz iron uptake by diaz
c     photoP_diaz  po4 uptake by diaz
c     photoP_sp    po4 uptake by small phyto
c     photoP_lp    po4 upatek by large phyto
c     po4_V_lp     P uptake rate by large phyto
c     po4_V_sp     P uptake rate by small phyto
c     po4_V_diaz   P uptake rate by diaz
c     Qcaco3_sp    small phyto Caco3/C ratio
c     ldetrQcaco3  large detritus Caco3/C ratio
c     z_slop_sp    portion grazed which is not added to Z biomass
c     z_slop_lp    portion grazed which is not added to Z biomass
c     z_slop_detr  portion grazed which is not added to Z biomass
c     LreminC      large detrital C remin
c     LreminN      large detrital N remin
c     LreminP      large detrital P remin
c     LreminFe     large detrital fe remin
c     LreminSi	   large detrital Si remin
c     LreminCaco3  large detrital CaCO3 remin
c     SreminC      small detrital C remin
c     SreminN      small detrital N remin
c     SreminP      small detrital P remin
c     SreminFe     small detrital Fe remin
c     Qnotca       proportion of small phyto not associated w/CaCO3
c     lambda       biosynthesis cost, modified by N source
c     frat_sp      f-ratio small phyto
c     frat_lp      f-ratio large phyto
c     ldetrQfe     large detritus fe/C ratio
c     sdetrQfe     small detritus fe/C ratio
c     graze_diaz   grazing on diazotrophs
c     z_slop_diaz  sloppy grazing on diazotrophs
c     diaz_kiron   half saturation constant for Fe uptake by diazotrophs
c     diaz_kp      half saturation constant for P uptake by diazotrophs

	real LreminC, LreminN, LreminP, LreminFe, LreminSi
	real LreminCaco3, SreminC, SreminN, SreminP,SreminFe
        real kpar,Epar,thetaC_sp,thetaN_sp,pChl, PCphoto_sp
        real thetaC_lp,thetaN_lp,PCphoto_lp, PCmax
        real Q_sp,Q_lp,Tfunc,Vnc_sp,Vnc_lp,Vncmax_sp,Vncmax_lp
        real f_fe_sp,Vsic_lp,ldetrQsi
        real Qfec_sp,Qfec_lp,f_nut,zooQfe
        real Vfec_sp,Vfec_lp,Vfecmax_sp,Vfecmax_lp,Vfecref,f_nit_sp
        real f_nit_lp,f_fe_lp,uQ_sp,uQfe_sp,uQ_lp,uQfe_lp,V1_sp
        real V2_sp,photoacclim_sp,V1_lp,V2_lp,photoacclim_lp
        real nitrif, zooQ,Zprime
        real sdetrQ,ldetrQ, f_si_lp,f_zoo_detr, graze_diaz,z_slop_diaz
        real fe_scavenge, Pprime, Qsi_lp,uQsi_lp,Vsicref,Vsicmax
        real amm_V_sp,no3_V_sp,amm_V_lp,no3_V_lp,reduceV
	real	thetaC_diaz,thetaN_diaz,PCphoto_diaz,Q_diaz
	real	Vncmax_diaz,diaz_Nexcrete, Qfec_diaz
	real    Vfec_diaz,Vfecmax_diaz,f_nit_diaz
	real    f_p_diaz,f_p_sp,f_p_lp, uQp_diaz,uQp_sp,uQp_lp
	real	photoacclim_diaz, Qp_sp, Qp_lp,Qp_diaz,zooQp
	real	sdetrQp, f_fe_diaz, lambda
	real	ldetrQp,Vpo4ref,Vpo4max,PCref_diaz,Vpo4ref_diaz
	real	Qpo4min, Qpo4max, Qpo4max_diaz,Qpo4min_diaz
	real    photoN_sp, photoN_lp,photoFe_lp, photoFe_sp
	real	photoSi_lp, uQ_diaz, Qfemax_diaz, Qfemin_diaz
	real	Vncref_diaz,photoFe_diaz,photoP_diaz, photoP_sp
	real    photoP_lp, po4_V_lp,po4_V_sp,po4_V_diaz
	real    Qcaco3_sp, ldetrQcaco3, z_slop_sp, z_slop_lp
	real	z_slop_detr, Qnotca, uQfe_diaz, Vfecref_diaz
	real    frat_sp,frat_lp, ldetrQfe, sdetrQfe
	real	diaz_kiron, Qfcaco3, diaz_kp
c
c  Fluxes saved to file, most fluxes saved as mixed layer totals (/m2)
c  These fluxes can be modified at will to look at different aspects of 
c  of the ecosystem. For example, focusing on carbon or iron cycling. 
c 
c
c 1    P quota diaz
c 2    Fe quota diaz
c 3    N quota diaz
c 4    Community F-ratio, (no3) / (no3 + NH4)  
c 5    graze_sp    grazing rate on small phyto, output as C units
c 6    graze_lp    grazing rate on large phyto, output as C units
c 8    lp_agg      large phytoplankton aggregation, output as C units
c 9    sp_agg      small phyto aggregation loss, output as C units
c 10   lp_loss     large phyto non-grazing mortality, output as C units
c 11   sp_loss     small phyto non-grazing mortality, output as C units
c 12   diaz_loss   diaz non-grazing mortality/ respiration, C units 
c 13   diaz_Nfix   total N-fixation by diazotrophs
c 14   graze_detr  grazing rate on large detritus, C units
c 15   photoC_diaz diaz C-fixation
c 16   photoC_sp   small phyto C-fixation
c 17   photoC_lp   large phyto C-fixation
c 18   community   F-ratio including N-fixation as new N
c 19   graze_diaz  total grazing on diazotrophs C units
c 20   CaCO3_prod  production of CaCO3 by phytoplankton
  
        real graze_lp,graze_sp,z_loss, diaz_loss, photoC_diaz
        real lp_agg,sp_agg,lp_loss,sp_loss, caco3_prod
        real photoC_lp,photoC_sp, diaz_Nfix, graze_detr


c     fluxes across base of ML, 
c   7   export of biogenic C out of mixed layer due to sinking
c  21   export_si  sinking export of Si within large detritus
c  22   export_caco3 of CaCO3 due to sinking of large detritus
c  23   export_sink export of Carbon due to sinking extrapolated to 150m  
c  24   export_turb export of Carbon due to turbulent mixing
c  25   export_detrain export of Carbon due to mixed layer shallowing       
c  26   export_upwell of Carbon  due to upwelling/dilution
c
c
c  27   P quota small phyto
c  28   N quota small phyto
c  29   Fe quota small phyto
c  30   N quota large phyto
c  31   Fe quota large phyto
c  32   Si quota large phyto 
c  33   P quota large phyto
c
c     fluxes across base of ML, 
c  34   export_pN   sinking export of particulate N within large detr.
c  35   export_pP   sinking export of particulate P within large detr.
c  36   export_pFe  sinking export of particulate Fe within large detr.
      
        real export_sink, export_turb, export_pN,export_pP
        real export_pFe, export_si, export_caco3,export_upw
        real export_detrain, importN, import_entrain

c     eps         machine zero
c     errflag     flag for printing error messages
        real eps
        integer k

        eps=1.0e-9
c***************************************************************************  
c     Load local variables and parameters
c     NOTE: The code does not check to see if you have inserted the correct
c     variables or parameter in the correct slot!!!! (i.e. do it yourself)

	no3 = x(1)
	amm = x(2)
	iron = x(3)
	silica = x(4)
	sphytoC = x(5)
	sphytoN = x(6)
	sphytoFe = x(7)
	sphytoChl = x(8)
	lphytoC = x(9)
	lphytoN = x(10)
	lphytoFe = x(11)
	lphytoChl = x(12)
	lphytoSi = x(13)
	zooC = x(14)
	zooN = x(15)
	zooFe = x(16)
	ldetrC = x(17)
	ldetrN = x(18)
	ldetrFe = x(19)
	ldetrSi = x(20)
	sdetrC = x(21)
	sdetrN = x(22)
	sdetrFe = x(23)
	po4 = x(24)
	sphytoP = x(25)
	lphytoP = x(26)
	zooP = x(27)
	ldetrP = x(28)
	sdetrP = x(29)
	diazC = x(30)
	diazN = x(31)
	diazFe = x(32)
	diazChl = x(33)
	diazP = x(34)
	sphytoCaco3 = x(35)
	ldetrCaco3 = x(36)
	

        Qmin = biopar(1)
        Qmax = biopar(2) 
        PCref = biopar(3)
        z_umax = biopar(4)
        lz_umax = biopar(5)
        sp_mort = biopar(6)
        sp_mort2 = biopar(7)
        lp_mort = biopar(8)
        lp_mort2 = biopar(9)
        z_mort = biopar(10)
        z_mort2 = biopar(11)
        sd_remin = biopar(12)
        ld_remin = biopar(13)
        ld_sink = biopar(14)
        sp_kno3 = biopar(15)
        lp_kno3 = biopar(16)
        sp_kamm = biopar(17)
        lp_kamm = biopar(18)
        sp_kiron = biopar(19)
        lp_kiron = biopar(20)
        lp_ksilica = biopar(21)
        z_grz = biopar(22)
        z_ingest = biopar(23)
        firon = biopar(24)
        diaz_mort = biopar(25)
        Vncref = biopar(26)
        thetaNmax = biopar(27)
        lambda_no3 = biopar(28)
        Qfemax = biopar(29)
        Qfemin = biopar(30)
        Qsimin = biopar(31)
        Qsimax = biopar(32)
        alphachl = biopar(33) 
	sp_kpo4 = biopar(34)
	lp_kpo4 = biopar(35)
	alphadiaz = biopar(36)

c  Initialize Diazotroph Parameters
c  max and min P/C ratios, set at Redfield, analogous to Qmin, Qmax for
c	large and small phyto, for diaz C/P = 298, and N/P = 45, 
c	iron requirements for diaz are 8 times greater than other phyto
	Qpo4max = 0.010625
	Qpo4min = 0.002125
	Qpo4max_diaz = 0.00378
	Qpo4min_diaz = 0.0007559
	PCref_diaz = 0.4
	diaz_kiron = sp_kiron
	diaz_kp = 2.0 * sp_kpo4
	dz_umax = 0.2

	Qfemax_diaz = 56.0
	Qfemin_diaz = 8.0	


c set local variables 
	dt_umax = 0.65 * lz_umax     
        thetaN_sp = sphytoChl / sphytoN
        thetaN_lp = lphytoChl / lphytoN
        thetaN_diaz = diazChl / diazN
        thetaC_sp = sphytoChl / sphytoC
        thetaC_lp = lphytoChl / lphytoC
        thetaC_diaz = diazChl / diazC
        Q_sp = sphytoN / sphytoC
        Q_lp = lphytoN / lphytoC
        Q_diaz = diazN / diazC
        Qfec_sp = sphytoFe / sphytoC
	Qfec_lp = lphytoFe / lphytoC
	Qfec_diaz = diazFe / diazC
	Qp_sp = sphytoP / sphytoC
	Qp_lp = lphytoP / lphytoC
	Qp_diaz = diazP / diazC
	Qsi_lp = lphytoSi / lphytoC
	
c Qfcaco3 is the percentage of organic matter which is associated
c   with coccoliths and is routed to large detr. when sp cells lyse
c   i.e. the non-grazing mortalitiy term, assumes inorganic C/ orgC ratio
c  of 0.5, and that 25% of this material is attached to Caco3, 
c   0.5*0.25=0.125
c QnotCa is the remainder which goes to the small detr pool
	Qcaco3_sp = sphytoCaco3 / sphytoC
	Qfcaco3 = Qcaco3_sp * 0.125
	QnotCa = (1.0 - Qfcaco3)
	
        zooQ = zooN / zooC
        zooQp = zooP / zooC
        zooQfe = zooFe / zooC
        sdetrQ = sdetrN / sdetrC
        ldetrQ = ldetrN / ldetrC
        sdetrQp = sdetrP / sdetrC
        ldetrQp = ldetrP / ldetrC
        ldetrQsi = ldetrSi / ldetrC
        ldetrQcaco3 = ldetrCaco3 / ldetrC
        ldetrQfe = ldetrFe / ldetrC
        sdetrQfe = sdetrFe / sdetrFe
        Vfecref = PCref * Qfemax
        Vpo4ref = PCref * Qpo4max
        Vpo4ref_diaz = PCref_diaz * Qpo4max_diaz
        Vncref_diaz = PCref_diaz * Qmax * 1.43
        Vfecref_diaz = PCref_diaz * Qfemax_diaz
        
        

	
c Boundary checks of acceptable values for nutrient uptake eqns.
        uQ_sp=min(Q_sp,Qmax)
  	uQ_sp=max(uQ_sp,Qmin)
        uQ_lp=min(Q_lp,Qmax)
  	uQ_lp=max(uQ_lp,Qmin)
  	uQ_diaz = min(Q_diaz,Qmax)
  	uQ_diaz = max(uQ_diaz,Qmin)
  	
	uQfe_sp = min(Qfec_sp,Qfemax)
	uQfe_sp = max(uQfe_sp,Qfemin)
	uQfe_lp = min(Qfec_lp,Qfemax)
	uQfe_lp = max(uQfe_lp,Qfemin)
	uQfe_diaz = min(Qfec_diaz,Qfemax_diaz)
	uQfe_diaz = max(uQfe_diaz,Qfemin_diaz)
	
        uQp_sp=min(Qp_sp,Qpo4max)
  	uQp_sp=max(uQp_sp,Qpo4min)
        uQp_lp=min(Qp_lp,Qpo4max)
  	uQp_lp=max(uQp_lp,Qpo4min)
  	uQp_diaz = min(Qp_diaz,Qpo4max_diaz)
  	uQp_diaz = max(uQp_diaz,Qpo4min_diaz)
	
 
c*************************************************************************
c     Compute biological flows (user specified)

c     photosynthetically available radiation (PAR) in W/m2
c     akw = 0.04        absorption coefficient for water (m)
c     akc = 0.03        absorption coefficient for chlorophyll (m)
c     fpar = 0.45       PAR as fraction of surface solar

        kpar = 0.04 + 0.03*(sphytoChl+lphytoChl+diazChl)
        Epar = 0.45*qsw/kpar/hmix*(1.0-exp(-kpar*hmix))

c set min. Epar, prevent any negative Epar values
	Epar = max(Epar,eps)
	
c     temperature dependence, Arrhenius relation (GD98)
c     Tref =303.15 reference temperature    (deg. K)     
c     AE = -4000.0 temp. coeff., slope of linear Arrhenius (GD98)
c     Tfunc = exp(-4000.0 *((1.0/(sst+273.15))-(1.0/303.15)))

         Tfunc = exp(-4000.0 *((1.0/(sst+273.15))- 0.0032987)) 

c  remineralization is a function of temperature
        sd_remin = sd_remin * Tfunc
        ld_remin = ld_remin * Tfunc

c  modify growth mortality rates by Tfunc        
        z_umax =  z_umax * Tfunc
	lz_umax = lz_umax * Tfunc
	dz_umax = dz_umax * Tfunc
	dt_umax = dt_umax * Tfunc
	z_mort =  z_mort * Tfunc
	
c  Under iron stress, ammonia preference increases, see META
c      1st get cellular iron quotas, (also used in Fe uptake eqns.)
	f_fe_sp = (uQfe_sp - Qfemin)/(Qfemax - Qfemin)
	f_fe_lp =(uQfe_lp - Qfemin)/(Qfemax - Qfemin)
	f_fe_diaz=(uQfe_diaz-Qfemin_diaz)/(Qfemax_diaz-Qfemin_diaz)
	
	
	sp_kno3 = sp_kno3 * min((1.0/(f_fe_sp+eps)),1.5)
	lp_kno3 = lp_kno3 * min((1.0/(f_fe_lp+eps)),1.5)
	sp_kamm = sp_kamm * max(f_fe_sp,0.5)
	lp_kamm = lp_kamm * max(f_fe_lp,0.5)
	
c under iron stress, cellular si demands also increase, see META
	Qsimax = Qsimax * min((1.0/(f_fe_lp+eps)), 2.0)
	uQsi_lp = min(Qsi_lp,Qsimax)
	uQsi_lp = max(uQsi_lp,Qsimin)
        Vsicref = PCref * Qsimax
         
        
c get N uptake rates for small phyto
c  	use Doney more gradual taper formula than GD98 
c	Vncmax is max. uptake for no3 & amm total

	f_nit_sp = (uQ_sp - Qmin) / (Qmax - Qmin)
	Vncmax_sp =Vncref*((1.0-f_nit_sp)/(1.015-f_nit_sp))*Tfunc

c get relative uptake rates nitrate is V1, ammonia is V2
	V1_sp = (no3/sp_kno3)/(1.0+(no3/sp_kno3)+(amm/sp_kamm))
	V2_sp = (amm/sp_kamm)/(1.0+(no3/sp_kno3)+(amm/sp_kamm))

c get Total N uptake by small phyto
	no3_V_sp = (V1_sp*Vncmax_sp/Q_sp) * sphytoN
	amm_V_sp = (V2_sp*Vncmax_sp/Q_sp) * sphytoN
	
c get small phyto f-ratio, used in computing biosynthesis cost
	frat_sp = no3_V_sp / (no3_V_sp + amm_V_sp + eps)
	
c     get N uptake for large phyto
c  use Doney more gradual taper formula, Vncmax is max. uptake for N
	f_nit_lp = (uQ_lp - Qmin) / (Qmax - Qmin)
	Vncmax_lp=Vncref*((1.0-f_nit_lp)/(1.015-f_nit_lp))*Tfunc

c Uptake rates for large phyto nitrate is V1, ammonia is V2
	V1_lp = (no3/lp_kno3)/(1.0+(no3/lp_kno3)+ (amm/lp_kamm))
	V2_lp = (amm/lp_kamm)/(1.0+(no3/lp_kno3)+ (amm/lp_kamm))

c get total N uptake by large phyto
	no3_V_lp = (V1_lp*Vncmax_lp/Q_lp) * lphytoN
	amm_V_lp = (V2_lp*Vncmax_lp/Q_lp) * lphytoN
	
c get small phyto f-ratio, used in computing biosynthesis cost
	frat_lp = no3_V_lp / (no3_V_lp + amm_V_lp + eps)


c     get small phyto total N uptake rate and N-based biomass change
	Vnc_sp = (V1_sp+V2_sp)*Vncmax_sp
	photoN_sp = no3_V_sp + amm_V_sp

c     get N-based large phyto total N uptake rate and biomass change
	Vnc_lp = (V1_lp + V2_lp) * Vncmax_lp
	photoN_lp = amm_V_lp + no3_V_lp
	
c     calculate nitrogen fixation for diazotrophs, supply of N2 unlimited
c     Diaz are assumed to fix 30% more N than needed, this 30% is excreted
c
	f_nit_diaz = (uQ_diaz - Qmin) / (Qmax - Qmin)
	Vncmax_diaz=Vncref_diaz*((1.0-f_nit_diaz)/
     &              (1.015-f_nit_diaz))*Tfunc
	diaz_Nfix = (Vncmax_diaz/Q_diaz) * diazN

	
	
c output total N fixation,
c	diaz_Nfix=0.0
	dxdt(nsb+13) = diaz_Nfix * hmix
	
c     output community N-based f-ratio
	dxdt(nsb+18)=(no3_V_lp+no3_V_sp+diaz_Nfix)/ 
     &	  (no3_V_lp+amm_V_lp+no3_V_sp+amm_V_sp+diaz_Nfix+eps)
	dxdt(nsb+4)=(no3_V_lp+no3_V_sp)/ 
     &	  (no3_V_lp+amm_V_lp+no3_V_sp+amm_V_sp+eps)
     
c divide Fixed Nitrogen into incorporated and excreted fractions
	diaz_Nexcrete = 0.3 * diaz_Nfix
	diaz_Nfix = 0.7 * diaz_Nfix
	
c     get Fe uptake for small phytoplankton
	Vfecmax_sp=Vfecref*((1.0-f_fe_sp)/(1.015-f_fe_sp))*Tfunc
	Vfec_sp=Vfecmax_sp*(iron/(iron + sp_kiron))

c     get Fe-based small phyto biomass change
	photoFe_sp = (Vfec_sp/Qfec_sp) * sphytoFe

c     get Fe uptake for large phytoplankton
	Vfecmax_lp=Vfecref*((1.0-f_fe_lp)/(1.015-f_fe_lp))*Tfunc
	Vfec_lp=Vfecmax_lp*(iron/(iron + lp_kiron))

c     get Fe-based large phyto biomass change
	photoFe_lp = (Vfec_lp/Qfec_lp) * lphytoFe

c     get Fe uptake for diaz
	Vfecmax_diaz=Vfecref_diaz*((1.0-f_fe_diaz)/
     &		(1.015-f_fe_diaz))*Tfunc
	Vfec_diaz=Vfecmax_diaz*(iron/(iron + diaz_kiron))

c     get Fe-based diaz biomass change
	photoFe_diaz = (Vfec_diaz/Qfec_diaz) * diazFe


c get P uptake for all phytoplankton groups	
c     get P uptake for small phytoplankton
	f_p_sp= (uQp_sp - Qpo4min) / (Qpo4max - Qpo4min)
	Vpo4max=Vpo4ref*((1.0-f_p_sp)/(1.015-f_p_sp))*Tfunc
	po4_V_sp=Vpo4max*(po4/(po4 + sp_kpo4))

c     get P-based small phyto biomass change
	photoP_sp = (po4_V_sp/Qp_sp) * sphytoP

c     get P uptake for large phytoplankton
	f_p_lp= (uQp_lp - Qpo4min) / (Qpo4max - Qpo4min)
	Vpo4max=Vpo4ref*((1.0-f_p_lp)/(1.015-f_p_lp))*Tfunc
	po4_V_lp=Vpo4max*(po4/(po4 + lp_kpo4))

c     get P-based large phyto biomass change
	photoP_lp = (po4_V_lp/Qp_lp) * lphytoP

c     get P uptake for diaz
	f_p_diaz=(uQp_diaz-Qpo4min_diaz)/(Qpo4max_diaz-Qpo4min_diaz)
	Vpo4max=Vpo4ref_diaz*((1.0-f_p_diaz)/
     &		(1.015-f_p_diaz))*Tfunc
	po4_V_diaz=Vpo4max*(po4/(po4 + diaz_kp))

c     get P-based diaz biomass change
	photoP_diaz = (po4_V_diaz/Qp_diaz) * diazP
	

c     get Si-based uptake for diatoms
	f_si_lp = (uQsi_lp - Qsimin) / (Qsimax - Qsimin)
	Vsicmax = Vsicref*((1.0 - f_si_lp)/(1.015-f_si_lp))*Tfunc
	Vsic_lp = Vsicmax * (silica/(silica + lp_ksilica)) 

c    get Si-based biomass change for diatoms
	photoSi_lp = (Vsic_lp/Qsi_lp) * lphytoSi
	dxdt(nsb+12) = photoSi_lp * hmix

c Small Phytoplankton C-fixation and photoadaptation	
c   determine small phyto nutrient limitation factor for carbon fixation
	f_nut= min(f_nit_sp,f_fe_sp)
	f_nut= min(f_nut,f_p_sp)

c    output limiting nutrient quotas 
	dxdt(nsb+27) = f_p_sp
	dxdt(nsb+28) = f_nit_sp
	dxdt(nsb+29) = f_fe_sp

c     get small phyto photosynthesis rate, phyto C biomass change, photoadapt
	lambda = lambda_no3 * max(frat_sp, 0.5)
     	PCmax = PCref * f_nut * Tfunc
     	PCphoto_sp =PCmax*(1.0-exp((-1.0*alphachl*thetaC_sp*Epar)/
     &    (PCmax+eps)))
     	photoC_sp = (PCphoto_sp-(lambda*Vnc_sp)) * sphytoC
     	
c     output primary production and instantaneous small phyto growth rate
	dxdt(nsb+16) = photoC_sp * hmix

c     calculate pChl, (used in photoadaptation, GD98)
	pChl = thetaNmax*PCphoto_sp/(alphachl*thetaC_sp*Epar)
	photoacclim_sp=((pChl*Vnc_sp)/thetaC_sp)*sphytoChl


	
c CaCO3 Production, parameterized as function of small phyto production
	caco3_prod = 0.05 * PCphoto_sp * sphytoC
c decrease caco3 as function of nutrient limitation
	caco3_prod = caco3_prod * f_nut * f_nut
c decrease caco3 prod at low temperatures
	if (sst.LT.5.0) caco3_prod = caco3_prod*(sst+2.0+eps)/28.0
	if (sst.LT.0.0) caco3_prod = caco3_prod*0.0001
c increase caco3 prod under bloom conditions
	if (sphytoC.GT.2.0) caco3_prod = caco3_prod*sphytoC/2.0
	dxdt(nsb+20) = caco3_prod * hmix



c Diatom carbon fixation and photoadaptation
c   determine large phyto nutrient limitation factor for carbon fixation
	f_nut= min(f_nit_lp,f_fe_lp)
	f_nut = min(f_nut,f_si_lp)
	f_nut = min(f_nut,f_p_lp)

c   output nutrient cell quotas
	dxdt(nsb+30) = f_nit_lp
	dxdt(nsb+31) = f_fe_lp
	dxdt(nsb+32) = f_si_lp
	dxdt(nsb+33) = f_p_lp

c   get large diatom photosynthesis rate, C biomass change, photoadaptation
	lambda = lambda_no3 * max(frat_lp, 0.5)
     	PCmax = PCref * f_nut * Tfunc
     	PCphoto_lp =PCmax*(1.0-exp((-1.0*alphachl*thetaC_lp*Epar)/
     &    (PCmax+eps)))
     	photoC_lp =(PCphoto_lp -(lambda*Vnc_lp)) * lphytoC
     	
c     output primary production
	dxdt(nsb+17) = photoC_lp * hmix

c calculate pChl and chl. synthesis terms (GD98)
	pChl = thetaNmax * PCphoto_lp/(alphachl*thetaC_lp*Epar)
	photoacclim_lp=((pChl*Vnc_lp)/thetaC_lp)*lphytoChl


c Diazotroph carbon fixation and photoadaptation
c   determine nutrient limitation factor
	f_nut = min(f_nit_diaz, f_fe_diaz)
	f_nut = min(f_nut, f_p_diaz)
	
c   output nutrient cell quotas	
	dxdt(nsb+1) = f_p_diaz
	dxdt(nsb+2) = f_fe_diaz
	dxdt(nsb+3) = f_nit_diaz
	
c   get diazotroph photosynthesis rate,
	PCmax = PCref_diaz * f_nut * Tfunc
     	PCphoto_diaz=PCmax*(1.0-exp((-1.0*alphadiaz*thetaC_diaz
     &       *Epar)/(PCmax+eps)))
     	photoC_diaz =(PCphoto_diaz -(lambda_no3*diaz_Nfix))*diazC
     	if (sst.LT.16.0) photoC_diaz= 0.0

c   output primary production
	dxdt(nsb+15) = photoC_diaz * hmix
		
c     calculate pChl, (used in photoadaptation, GD98)
	pChl=thetaNmax*PCphoto_diaz/(alphadiaz*thetaC_diaz*Epar)
	photoacclim_diaz=((pChl*diaz_Nfix)/thetaC_diaz)*diazChl



c CALCULATE GRAZING AND OTHER MORTALITY
C
c Grazing Losses - z_grz is set lower for diatoms and large detritus
c  	than for small phytoplankton and diazotrophs, see META

c  get grazing rate (graze_sp) on small phyto  (in C units)
	reduceV = sphytoC*sphytoC
	z_grz = z_grz * z_grz
	graze_sp = z_umax*zooC*(reduceV/(reduceV + z_grz))
        dxdt(nsb+5) = graze_sp* hmix
                 
c  get grazing rate on large detritus
	reduceV = ldetrC * ldetrC
	graze_detr= dt_umax*zooC*(reduceV/(reduceV+z_grz*0.81))
	dxdt(nsb+14) = graze_detr*hmix
	
c  get grazing rate on diazotrophs
	reduceV = diazC * diazC
	graze_diaz = dz_umax*zooC*(reduceV/(reduceV+z_grz))
	dxdt(nsb+19) = graze_diaz * hmix

c  get grazing rate (graze_lp) on diatoms  (in C units)
	reduceV = lphytoC*lphytoC
	graze_lp = lz_umax*zooC*(reduceV/(reduceV + z_grz*0.81))
        dxdt(nsb+6) = graze_lp* hmix


	
c Non-grazing Losses - 
c     get small phyto loss(in C units), mortality ->0,as biomass gets low
	Pprime = max((sphytoC - 0.001),0.0)
        sp_loss = sp_mort * Pprime 
        dxdt(nsb+11) = sp_loss * hmix
        
c     small phyto aggregation loss,
        sp_agg = sp_mort2*Pprime*Pprime/256.0
        sp_agg = min((0.7*Pprime), sp_agg)
        dxdt(nsb+9) = sp_agg * hmix

c     get large phyto loss(in C units),mortality ->0,as biomass gets low
	Pprime = max((lphytoC - 0.005),0.0)
        lp_loss = lp_mort * Pprime 
        dxdt(nsb+10) = lp_loss * hmix
        
c     diatom aggregation/sinking loss, min. 5%/day
     	lp_agg=lp_mort2*Pprime*Pprime/256.0
     	lp_agg=max(lp_agg, (0.05*Pprime))
     	lp_agg=min((0.7*Pprime), lp_agg)
        dxdt(nsb+8) = lp_agg * hmix
        
c    get diaz mortality/respiration loss, min. biomass lower at sst < 16.0
	Pprime = max((diazC - 0.03), 0.0)
     	if (sst.LT.16.0) Pprime = max((diazC - 0.001),0.0)
	diaz_loss = diaz_mort*Pprime
	
	
c  get fractional factor for routing or zooplankton losses, based on 
c    current food supply, see META
	f_zoo_detr=((0.8*graze_lp)+(0.3*graze_sp)+(0.8*graze_detr)
     &  +(0.5*graze_diaz))/(graze_lp+graze_sp+graze_detr+graze_diaz)

c     zooplankton sloppy feeding  ( in C units!)
        z_slop_sp = (1.0 - z_ingest) * graze_sp
        z_slop_lp = (1.0 - z_ingest) * graze_lp
        z_slop_detr = (1.0 - z_ingest) * graze_detr
        z_slop_diaz = (1.0 - z_ingest) * graze_diaz
       
c     zooplankton losses Zprime used to set min. biomass
	Zprime = max((zooC - 0.01),0.0)
        z_loss = z_mort*Zprime*Zprime+z_mort2*Zprime
        
c  detrital remineralization with variable elemental remineralization rates 
        SreminN  = sdetrN  * sd_remin
        SreminP  = sdetrP  * sd_remin
        SreminFe = sdetrFe * sd_remin
        SreminC  = sdetrC  * sd_remin * 0.95
                
        LreminN  = ldetrN  * ld_remin
        LreminP  = ldetrP  * ld_remin
        LreminFe = ldetrFe * ld_remin
        LreminC  = ldetrC  * ld_remin * 0.95
        LreminSi = ldetrSi * ld_remin * 0.5
        LreminCaco3 = ldetrCaco3 * ld_remin * 0.01


c  set nitrification, only occurs at low light 
	nitrif = 0.0
	if (Epar.LT.4.0) nitrif = 0.04 * amm
	
c  iron adsorption/scavening, increased scavenging at higher iron levels,
c  min. 1%/year (0.0000274/day) at iron < 0.6 nM, 
	fe_scavenge= 0.0000274 * iron
	if (iron.GT.600.0) then
	fe_scavenge = 0.01644 + (0.0274 * (iron - 600.0) * 
     &          ((iron-600.0) / (iron + 1400.0)))
	endif



c Calculate scalar in situ rates of change (per day)

c nitrate
        dxdt(1) = nitrif - no3_V_lp - no3_V_sp
        
c ammonia
	dxdt(2) = -amm_V_lp-amm_V_sp+(z_slop_lp*Q_lp*0.5)
     &      + SreminN + LreminN-nitrif+(z_slop_detr*ldetrQ*0.5)
     &      +(z_slop_sp*Q_sp*0.5)+(z_slop_diaz*Q_diaz*0.5)

c dissolved iron  
        dxdt(3) = -photoFe_sp -photoFe_lp +LreminFe -photoFe_diaz
     &	 +SreminFe-fe_scavenge+(z_slop_lp*Qfec_lp*0.5)
     &   +(feflux*firon/hmix)+(z_slop_sp*Qfec_sp*0.5)
     &   +(graze_detr*ldetrQfe*0.35)+(z_slop_diaz*Qfec_diaz*0.5)
             
c dissolved silicate
c    atoms. Si flux calculated as 30.8% Si by wt./3.5% Fe by wt. * 7.5% 
c      solubility * fe flux * 1.0e-6 to convert from nmol iron to mmol Si, 
c      see META for details and references.

	dxdt(4) = - photoSi_lp+LreminSi+(0.35*graze_lp*Qsi_lp)
     &		+ (0.35*graze_detr*ldetrQsi)
     &		+ (feflux*0.00000066/hmix)
     
c small phyto Carbon
      	dxdt(5)= photoC_sp - graze_sp - sp_loss -sp_agg

c small phyto N
        dxdt(6) = photoN_sp - ((sp_loss+sp_agg+graze_sp)*Q_sp)

c small phyto Fe
	dxdt(7) = photoFe_sp -((sp_loss+sp_agg+graze_sp)*Qfec_sp)   
     
c small phyto chlorophyll
        dxdt(8) = photoacclim_sp - 
     &            ((graze_sp+sp_loss+sp_agg)*thetaC_sp)
     
c diatom Carbon
      	dxdt(9)= photoC_lp - graze_lp - lp_loss -lp_agg

c diatom N
        dxdt(10) = photoN_lp -((graze_lp+lp_loss+lp_agg)*Q_lp)

c diatom Fe
	dxdt(11) = photoFe_lp -((graze_lp+lp_loss+lp_agg)*Qfec_lp)  
     
c diatom chlorophyll
        dxdt(12) = photoacclim_lp - 
     &             ((graze_lp+lp_loss+lp_agg)*thetaC_lp)

c diatom Si
	dxdt(13) = photoSi_lp -((graze_lp+lp_loss+lp_agg)*Qsi_lp)

c zooplankton Carbon
	dxdt(14) = (graze_sp+graze_lp+graze_detr+graze_diaz)
     &             *z_ingest - z_loss
        
c zooplankton N
        dxdt(15)=(z_ingest*graze_lp*Q_lp)+(z_ingest*graze_sp*Q_sp)
     &       +(z_ingest*graze_detr*ldetrQ) - (z_loss*zooQ)
     &	     + (z_ingest*graze_diaz*Q_diaz)
        
c zooplankton Fe
	dxdt(16) = (z_ingest*graze_lp*Qfec_lp) +
     &    (z_ingest*graze_sp*Qfec_sp) 
     &      + (z_ingest*graze_detr*ldetrQfe) 
     &     -(z_loss*zooQfe)+(z_ingest*graze_diaz*Qfec_diaz)

c when detritus is grazed, 30% goes to Zoo, 35% is remin, 35% remains detr
c     for Si and CaCO3, 65% remains in detr, 35% is remin

c large detritus C
	dxdt(17) =sp_agg+(sp_loss*Qfcaco3)+
     &          + (f_zoo_detr *z_loss)-LreminC
     &           + z_slop_lp*0.5+lp_agg+lp_loss*0.25
     &		+ (z_slop_diaz*0.25) - graze_detr*0.65

c large detritus N
        dxdt(18) =((lp_agg+lp_loss*0.25+z_slop_lp*0.5)*Q_lp)
     &		+((sp_agg+sp_loss*Qfcaco3)*Q_sp)
     &    	+(f_zoo_detr*z_loss*zooQ) -LreminN
     &      +(z_slop_diaz*Q_diaz*0.25) -(graze_detr*ldetrQ*0.65)

c large detritus Fe
	dxdt(19) = (f_zoo_detr*z_loss*zooQfe) - LreminFe
     &   +fe_scavenge -(graze_detr*ldetrQfe*0.65)
     &   +((z_slop_lp*0.5+lp_agg+0.25*lp_loss)*Qfec_lp)
     &   + ((sp_agg+(sp_loss*Qfcaco3))*Qfec_sp)
     &   + (z_slop_diaz*Qfec_diaz*0.25)
  

c large detritus Si
	dxdt(20) =-LreminSi+((0.65*graze_lp+lp_agg+lp_loss)*Qsi_lp)
     &		-(0.35*graze_detr*ldetrQsi)

c small detritus C
	dxdt(21) = sp_loss*QnotCa - SreminC + diaz_loss
     &    +((1.0-f_zoo_detr)*z_loss)+z_slop_sp*0.5
     &	  +lp_loss*0.75+ z_slop_diaz*0.25

c small detritus N
        dxdt(22)= -SreminN +(diaz_loss*Q_diaz)
     &    +((1.0-f_zoo_detr)*z_loss*zooQ) + diaz_Nexcrete
     &    +((z_slop_sp*0.5+sp_loss*QnotCa)*Q_sp)+(lp_loss*Q_lp*0.75)
     &    +(z_slop_diaz*Q_diaz*0.25)

c small detritus Fe
	dxdt(23) = ((sp_loss*QnotCa+ z_slop_sp*0.5)*Qfec_sp)
     &            + ((1.0-f_zoo_detr)*z_loss*zooQfe)
     &     - SreminFe + ((z_slop_diaz*0.25+diaz_loss)*Qfec_diaz)
     &	   + (lp_loss*Qfec_lp*0.75) 
     
c phosphate
	dxdt(24)= LreminP + SreminP - photoP_sp - photoP_lp
     &     -photoP_diaz+(z_slop_sp*Qp_sp*0.5)
     &     + (z_slop_lp*Qp_lp*0.5) + (z_slop_detr*ldetrQp*0.5)
     &     + (z_slop_diaz*Qp_diaz*0.5)
     
c small phyto P
	dxdt(25)= photoP_sp - ((graze_sp+sp_loss+sp_agg)*Qp_sp)
     
c large phyto P
	dxdt(26)= photoP_lp - ((graze_lp+lp_loss+lp_agg)*Qp_lp)
     
c zoopl P
	dxdt(27)=(z_ingest*graze_lp*Qp_lp)+(z_ingest*graze_sp*Qp_sp)
     &          +(z_ingest*graze_detr*ldetrQp) -(z_loss*zooQp)
     &	        +(z_ingest*graze_diaz*Qp_diaz)
     
c large detritus P
	dxdt(28)=((lp_agg+(lp_loss*0.25)+(z_slop_lp*0.5))*Qp_lp)
     &	   + ((sp_agg+sp_loss*Qfcaco3)*Qp_sp)
     &    +(f_zoo_detr*z_loss*zooQp)-LreminP
     &    +(z_slop_diaz*Qp_diaz*0.25)-(graze_detr*ldetrQp*0.65)
     
c small detritus P
	dxdt(29)= ((sp_loss*QnotCa+z_slop_sp*0.5)*Qp_sp) 
     &     +((1.0-f_zoo_detr)*z_loss*zooQp)
     &     +(lp_loss*Qp_lp*0.75) -SreminP
     &     +((z_slop_diaz*0.25+diaz_loss)*Qp_diaz)
	
c diaz C
	dxdt(30)= photoC_diaz - diaz_loss - graze_diaz
	
c diaz N
	dxdt(31)= diaz_Nfix - ((diaz_loss+graze_diaz)*Q_diaz)
	
c diaz Fe
	dxdt(32)= photoFe_diaz-((diaz_loss+graze_diaz)*Qfec_diaz)
	
c diaz Chl
	dxdt(33)= photoacclim_diaz- ((diaz_loss+graze_diaz)
     &		    *thetaC_diaz)
	
c diaz P
	dxdt(34)= photoP_diaz -((diaz_loss+graze_diaz)*Qp_diaz)
	
c small phytoplankton CaCO3
	dxdt(35)= caco3_prod -((graze_sp+sp_loss+sp_agg)*Qcaco3_sp)

c large detrital CaCO3, 35% of caco3 is remin when detritus is grazed
c    50% of caco3 is remin when phyto are grazed
	dxdt(36)= ((0.5*graze_sp+sp_loss+sp_agg)*Qcaco3_sp)
     &         -LreminCaco3-(0.35*graze_detr*ldetrCaco3/ldetrC)
	

c***************************************************************************
c     Bottom boundary conditions and fluxes
c     the rate of change equals the flux divided by depth of mixed layer
c     For current concentrations use initial + 0.5* dxdt, thus getting
c     average concentration over this time step
c

c Large detrital sinking, maximum sinking loss is all ldetr C
	export_sink=min((ld_sink/hmix*(ldetrC+0.5*dxdt(17))),
     &       (ldetrC+0.5*dxdt(17)))
        export_sink=max(export_sink, 0.0)
	
c recalculate large detritus ratios based on updated pools
	ldetrQfe =(ldetrFe+0.5*dxdt(19))/(ldetrC+0.5*dxdt(17))
	ldetrQ   = (ldetrN+0.5*dxdt(18)) / (ldetrC+0.5*dxdt(17))
	ldetrQsi = (ldetrSi+0.5*dxdt(20)) / (ldetrC+0.5*dxdt(17))
	ldetrQcaco3=(ldetrCaco3+0.5*dxdt(36))/(ldetrC+0.5*dxdt(17))
	ldetrQp  = (ldetrP+0.5*dxdt(28))/(ldetrC+0.5*dxdt(17))

	export_si    = max(export_sink*ldetrQsi, 0.0)
	export_pN    = max(export_sink*ldetrQ, 0.0)
	export_pP    = max(export_sink*ldetrQp, 0.0)
	export_pFe   = max(export_sink*ldetrQfe,0.0)
	export_caco3 = max(export_sink*ldetrQcaco3,0.0)

	dxdt(17) = dxdt(17) - export_sink
	dxdt(18) = dxdt(18) - export_pN
	dxdt(19) = dxdt(19) - export_pFe
	dxdt(20) = dxdt(20) - export_si
	dxdt(28) = dxdt(28) - export_pP
	dxdt(36) = dxdt(36) - export_caco3

c output sinking fluxes, base of mixed layer	
	dxdt(nsb+21) = export_si * hmix
	dxdt(nsb+22) = export_caco3 * hmix	
	dxdt(nsb+34) = export_pN * hmix
	dxdt(nsb+35) = export_pP * hmix
	dxdt(nsb+36) = export_pFe * hmix
	dxdt(nsb+7)  = export_sink * hmix


c  extrapolate sinking flux to 150m, allowing for variable remineralization
        export_sink=max((export_sink*hmix-(max(150.0-hmix,0.0)
     &           *0.75/ld_sink*ld_remin*0.95*export_sink*hmix)),0.0)

	dxdt(nsb+23) = export_sink

	
		
	
c     turbulent fluxes at base of ML
        export_turb=0.0
        do k=1,nsb
          dxdt(k)=dxdt(k)+(wturb*(bbc(k)-(x(k)+0.5*dxdt(k)))/hmix)
            
c   sum turbulent mixing of biogenic C out of mixed layer
          if ((k.EQ.5).OR.(k.EQ.9).OR.(k.EQ.14).OR.(k.EQ.17)
     &           .OR.(k.EQ.21).OR.(k.EQ.30))
     &       export_turb=export_turb+wturb*((x(k)+0.5*dxdt(k))-bbc(k))      		
        enddo        

        
c     entrainment and detrainment

c     detrainment does not alter mixed layer concentrations (dhdt < 0.0)

	export_detrain=0.0
	import_entrain=0.0
	importN = 0.0

c     entrainment (dhdt > 0.)        
        if(dhdt.gt.0) then
           do k=1,nsb
             dxdt(k)=dxdt(k)+(dhdt*(bbc(k)-(x(k)+0.5*dxdt(k)))/hmix)

c  sum entrainment of biogenic carbon into mixed layer
         if ((k.EQ.5).OR.(k.EQ.9).OR.(k.EQ.14).OR.(k.EQ.17)
     &     .OR.(k.EQ.21).OR.(k.EQ.30))
     &      import_entrain= import_entrain + dhdt*bbc(k)   

           enddo
	   importN = importN + dhdt*bbc(1)
	   
        else        
c     detrainment (dhdt < 0.)

         do k=1,nsb
         
c  sum detrainment of biogenic carbon out of mixed layer
         if ((k.EQ.5).OR.(k.EQ.9).OR.(k.EQ.14).OR.(k.EQ.17)
     &     .OR.(k.EQ.21).OR.(k.EQ.30))
     &     export_detrain=export_detrain-(dhdt*(x(k)+0.5*dxdt(k)))      		
         enddo
         
         
        endif
        
c  vertical velocity at base of mixed layer, if wvel < 0.0 (downwelling)
c    mixed layer concentrations do not change, assume lateral transport
c    of water with identical concentrations replaces downwelled water
c    if wvel > 0.0 (upwelling), water is assumed pushed out to side 
c      thus the change in mixed layer total budget is wvel*(bbc-x),
c    effect on mixed layer concentrations is same as entrainment.
c   upwelling results in a decrease of total mixed layer biogenic C, 
c   as upwelled < mixed layer.  The material exported laterally is not
c   tracked in the model and thus represents a loss to the ML (META, METB).
  
	export_upw = 0.0
	if (wvel.gt.0.0) then 

	   do k=1,nsb
	     dxdt(k)=dxdt(k)+(wvel*(bbc(k)-(x(k)+0.5*dxdt(k)))/hmix)

c   sum biogenic C dilution 
        if ((k.EQ.5).OR.(k.EQ.9).OR.(k.EQ.14).OR.(k.EQ.17)
     &     .OR.(k.EQ.21).OR.(k.EQ.30))
     &    export_upw=export_upw+(wvel*((x(k)+0.5*dxdt(k))-bbc(k))) 

	   enddo
	   
	endif

        dxdt(nsb+24)=export_turb   
   	dxdt(nsb+25)=export_detrain
   	dxdt(nsb+26)=export_upw
   	


        RETURN
        END





